@@ -3,95 +3,95 @@
 ##
 
 #' @title Create a graph object of class loongraph
-#'   
-#' @description The loongraph class provides a simple alternative to the graph 
+#'
+#' @description The loongraph class provides a simple alternative to the graph
 #'   class to create common graphs that are useful for use as navigation graphs.
-#'   
-#'   
+#'
+#'
 #' @param nodes a character vector with node names, each element defines a node
 #'   hence the elements need to be unique
 #' @param from a character vector with node names, each element defines an edge
 #' @param to  a character vector with node names, each element defines an edge
 #' @param isDirected boolean scalar, defines whether from and to define directed
 #'   edges
-#'   
-#'   
+#'
+#'
 #' @details loongraph objects can be converted to graph objects (i.e. objects of
-#' class graph which is defined in the graph package) with the as.graph 
+#' class graph which is defined in the graph package) with the as.graph
 #' function.
-#' 
+#'
 #' @templateVar page learn_R_display_graph.html
 #' @templateVar section graph-utilities
 #' @template see_l_help
 #'
-#' @template return_loongraph 
-#' 
-#' @seealso \code{\link{completegraph}}, \code{\link{linegraph}}, 
+#' @template return_loongraph
+#'
+#' @seealso \code{\link{completegraph}}, \code{\link{linegraph}},
 #'   \code{\link{complement}}, \code{\link{as.graph}}
-#'   
+#'
 #' @export
-#'      
-#' @examples 
+#'
+#' @examples
 #' g <- loongraph(
 #'   nodes = c("A", "B", "C", "D"),
 #'   from = c("A", "A", "B", "B", "C"),
 #'   to   = c("B", "C", "C", "D", "D")
 #' )
-#' 
+#'
 #' \dontrun{
 #' # create a loon graph plot
 #' p <- l_graph(g)
 #' }
-#' 
+#'
 #' lg <- linegraph(g)
 loongraph <- function(nodes, from=character(0), to=character(0), isDirected=FALSE) {
 
     if (length(nodes) != length(unique(nodes)))
         warning("node names are not unique")
-    
+
     if (length(from) != length(to))
         stop(paste0("'from' and 'to' differ in length: ",
                     length(from), ' vs. ', length(to)))
-    
+
     if (length(from) != 0  && !all(from %in% nodes))
         stop(paste("the following nodes in 'from' do not exist:",
                    paste(from[!(from %in% nodes)], collapse=', ')))
-    
+
     if (length(to) != 0 && !all(to %in% nodes))
         stop(paste("the following nodes in 'to' do not exist:",
                    paste(to[!(to %in% nodes)], collapse=', ')))
 
     graph <- list(nodes=nodes, from=from, to=to, isDirected=isDirected)
-    
+
     class(graph) <- "loongraph"
-    
+
     return(graph)
 }
 
 
 #' @title Convert a graph object to a loongraph object
-#'   
+#'
 #' @description Sometimes it is simpler to work with objects of class loongraph
 #'   than to work with object of class graph.
-#' 
+#'
 #' @param graph object of class graph (defined in the graph library)
-#' 
-#' 
-#' @details See 
-#'   \url{http://www.bioconductor.org/packages/release/bioc/html/graph.html} for
+#'
+#'
+#' @details See
+#'   \url{https://www.bioconductor.org/packages/release/bioc/html/graph.html} for
 #'   more information about the graph R package.
-#'   
+#'
 #' @templateVar page learn_R_display_graph.html
 #' @templateVar section graph-utilities
 #' @template see_l_help
-#'   
-#'   
-#' @template return_loongraph 
-#' 
-#' 
+#'
+#'
+#' @template return_loongraph
+#'
+#'
 #' @export
-#'   
-#' @examples 
+#'
+#' @examples
 #' if (requireNamespace("graph", quietly = TRUE)) {
 #'   graph_graph  = graph::randomEGraph(LETTERS[1:15], edges=100)
 #'   loon_graph <- as.loongraph(graph_graph)
@@ -99,46 +99,46 @@ loongraph <- function(nodes, from=character(0), to=character(0), isDirected=FALS
 as.loongraph <- function(graph) {
     if (!is(graph, "graph")) stop("graph argument is not of class graph.")
     requireNamespace("graph", quietly = TRUE) || stop("graph package is required for this function.")
-    
+
     nodes <- graph::nodes(graph)
     ft <- graph::edgeMatrix(graph)
-    
+
     loongraph(nodes=nodes, from=nodes[ft[1,]], to=nodes[ft[2,]],
-              isDirected=graph::isDirected(graph))    
+              isDirected=graph::isDirected(graph))
 }
 
 
 #' @title Convert a loongraph object to an object of class graph
-#'   
+#'
 #' @description Loon's native graph class is fairly basic. The graph package (on
-#'   bioconductor) provides a more powerful alternative to create and work with 
-#'   graphs. Also, many other graph theoretic algorithms such as the complement 
-#'   function and some graph layout and visualization methods are implemented 
-#'   for the graph objects in the RBGL and Rgraphviz R packages. For more 
+#'   bioconductor) provides a more powerful alternative to create and work with
+#'   graphs. Also, many other graph theoretic algorithms such as the complement
+#'   function and some graph layout and visualization methods are implemented
+#'   for the graph objects in the RBGL and Rgraphviz R packages. For more
 #'   information on packages that are useful to work with graphs see the
 #'   \emph{gRaphical Models in R} CRAN Task View at
 #'   \url{https://CRAN.R-project.org/view=gR}.
-#'   
+#'
 #' @param loongraph object of class loongraph
-#'   
-#' @details See 
-#'   \url{http://www.bioconductor.org/packages/release/bioc/html/graph.html} for
+#'
+#' @details See
+#'   \url{https://www.bioconductor.org/packages/release/bioc/html/graph.html} for
 #'   more information about the graph R package.
-#'   
+#'
 #' @template return_loongraph
-#'   
+#'
 #' @export
-#' 
-#' @examples 
+#'
+#' @examples
 #' if (requireNamespace("graph", quietly = TRUE)) {
 #'   g <- loongraph(letters[1:4], letters[1:3], letters[2:4], FALSE)
-#'   g1 <- as.graph(g) 
+#'   g1 <- as.graph(g)
 #' }
 as.graph <- function(loongraph) {
     if (!is(loongraph, "loongraph")) {
         stop("loongraph argument is not of class loongraph.")
     }
-    
+
     if (!requireNamespace("graph", quietly = TRUE)) {
         stop("graph package needed for this function to work. Please install it from bioconductor.",
              call. = FALSE)
@@ -154,7 +154,7 @@ as.graph <- function(loongraph) {
         }
     } else {
         for(i in 1:n) {
-            
+
             edL[[i]] <- unique(c(loongraph$to[loongraph$from==loongraph$nodes[i]],
                           loongraph$from[loongraph$to==loongraph$nodes[i]]))
         }
@@ -163,58 +163,58 @@ as.graph <- function(loongraph) {
     new("graphNEL", nodes = loongraph$nodes,
         edgeL=edL,
         edgemode= ifelse(loongraph$isDirected,"directed","undirected"))
-    
+
 }
 
 
 #' @title Plot a loon graph object with base R graphics
-#'   
-#' @description This function converts the loongraph object to one of class 
+#'
+#' @description This function converts the loongraph object to one of class
 #'   graph and the plots it with its respective plot method.
-#'   
+#'
 #' @param x object of class loongraph
 #' @param ... arguments forwarded to method
-#' 
+#'
 #' @export
-#' 
-#' @examples 
+#'
+#' @examples
 #' if (requireNamespace("Rgraphviz", quietly = TRUE)) {
 #'   g <- loongraph(letters[1:4], letters[1:3], letters[2:4], FALSE)
 #'   plot(g)
 #' }
 plot.loongraph <- function(x, ...) {
-    
+
     requireNamespace("Rgraphviz", quietly = TRUE) || stop("Rgraphviz library required")
-    
+
     Rgraphviz::plot(as.graph(x), ...)
 }
 
 
 #' @title Create a complete graph or digraph with a set of nodes
-#'   
-#' @description From Wikipedia: "a complete graph is a simple undirected graph 
-#'   in which every pair of distinct vertices is connected by a unique edge. A 
-#'   complete digraph is a directed graph in which every pair of distinct 
+#'
+#' @description From Wikipedia: "a complete graph is a simple undirected graph
+#'   in which every pair of distinct vertices is connected by a unique edge. A
+#'   complete digraph is a directed graph in which every pair of distinct
 #'   vertices is connected by a pair of unique edges (one in each direction
-#'   
+#'
 #' @inheritParams loongraph
-#' @param isDirected a boolean scalar to indicate wheter the returned object is 
+#' @param isDirected a boolean scalar to indicate wheter the returned object is
 #'   a complete graph (undirected) or a complete digraph (directed).
-#'   
-#'         
+#'
+#'
 #' @details Note that this function masks the completegraph function of the
 #'   graph package. Hence it is a good idead to specify the package namespace
 #'   with ::, i.e. loon::completegraph and graph::completegraph.
-#'   
+#'
 #' @templateVar page learn_R_display_graph.html
 #' @templateVar section graph-utilities
 #' @template see_l_help
-#'   
+#'
 #' @template return_loongraph
-#' 
+#'
 #' @export
-#'   
-#' @examples 
+#'
+#' @examples
 #' g <- loon::completegraph(letters[1:5])
 completegraph <- function(nodes, isDirected=FALSE) {
 
@@ -237,17 +237,17 @@ completegraph <- function(nodes, isDirected=FALSE) {
 
 
 #' @title Create a linegraph
-#' 
-#' @description The line graph of G, here denoted L(G), is the graph whose nodes 
-#'   correspond to the edges of G and whose edges correspond to nodes of G such 
-#'   that nodes of L(G) are joined if and only if the corresponding edges of G 
+#'
+#' @description The line graph of G, here denoted L(G), is the graph whose nodes
+#'   correspond to the edges of G and whose edges correspond to nodes of G such
+#'   that nodes of L(G) are joined if and only if the corresponding edges of G
 #'   are adjacent in G.
-#'   
+#'
 #' @param x graph of class graph or loongraph
 #' @param ... arguments passed on to method
-#'   
+#'
 #' @return graph object
-#'   
+#'
 #' @export
 linegraph <- function(x, ...) {
     UseMethod("linegraph")
@@ -255,37 +255,37 @@ linegraph <- function(x, ...) {
 
 
 #' @title Create a linegraph of a graph
-#'   
+#'
 #' @description Create a lingraph of a loongraph
-#'   
+#'
 #' @param x loongraph object
-#' @param separator one character - node names in x get concatenated with this 
+#' @param separator one character - node names in x get concatenated with this
 #'   character
 #' @template param_dots_method_not_used
-#'   
+#'
 #' @details linegraph.loongraph needs the code part for directed graphs (i.e.
 #'   isDirected=TRUE)
-#'   
+#'
 #' @template return_loongraph
-#'   
+#'
 #' @export
-#' 
-#' @examples 
+#'
+#' @examples
 #' g <- loongraph(letters[1:4], letters[1:3], letters[2:4], FALSE)
-#' 
+#'
 #' linegraph(g)
 linegraph.loongraph <- function(x, separator=":", ...) {
     nodes <- x$nodes
     from <- x$from
     to <- x$to
-    
-    n <- length(nodes) 
+
+    n <- length(nodes)
     p <- length(from)
-    
+
     if (x$isDirected) {
 
         stop("not implemented for directed graphs yet.")
-       
+
 #        foreach nfrom1 $from nto1 $to {
 #            foreach nfrom2 $from nto2 $to {
 #                if {!($nfrom1 eq $nfrom2 && $nto1 eq $nto2)} {
@@ -301,12 +301,12 @@ linegraph.loongraph <- function(x, separator=":", ...) {
 #            }
 #        }
 
-        
+
     } else {
 
         newfrom <- character(0)
         newto <- character(0)
-        
+
         for (i in 1:p) {
             for (j in i:p) {
                 if (i!=j && !(from[i] == from[j] && to[i] == to[j])) {
@@ -320,41 +320,41 @@ linegraph.loongraph <- function(x, separator=":", ...) {
             }
         }
     }
-    
+
     newnodes <- unique(c(newfrom, newto))
 
     G <- loongraph(nodes=newnodes, from=newfrom,
                    to=newto, isDirected=x$isDirected)
-    
+
     attr(G, "separator") <- separator
-    
+
     return(G)
 }
 
 #' @title Create the Complement Graph of a Graph
-#' 
+#'
 #' @description Creates a complement graph of a graph
-#' 
+#'
 #' @param x graph or loongraph object
-#' 
+#'
 #' @return graph object
-#' 
+#'
 #' @export
 complement <- function(x) {
-    UseMethod("complement", x)
+    UseMethod("complement")
 }
 
 
 #' @title Create the Complement Graph of a loon Graph
-#'   
+#'
 #' @description Creates a complement graph of a graph
-#'   
+#'
 #' @details This method is currently only implemented for undirected graphs.
-#'   
+#'
 #' @param x loongraph object
-#'   
+#'
 #' @template return_loongraph
-#'   
+#'
 #' @export
 complement.loongraph <- function(x) {
     nodes <- x$nodes
@@ -364,9 +364,9 @@ complement.loongraph <- function(x) {
 
     newfrom <- character(0)
     newto <- character(0)
-    
+
     if (x$isDirected) {
-        
+
         stop("not implemented for directed graphs yet.")
 
     } else {
@@ -384,11 +384,11 @@ complement.loongraph <- function(x) {
         }
     }
 
-    
-    
+
+
     G <- loongraph(nodes=nodes, from=newfrom, to=newto,
                    isDirected=x$isDirected)
-    
+
     attributes(G) <- attributes(x)
     return(G)
 }
@@ -401,12 +401,12 @@ complement.loongraph <- function(x) {
 
 
 # Create a graph product
-# 
+#
 # TODO: graphproduct is incomplete
 graphproduct <- function(U,V, type=c("product", "tensor", "strong"), separator=':') {
 
     stop("not implemented yet.")
-    
+
     type <- match.arg(type)
 
     switch(type,
@@ -424,40 +424,40 @@ graphproduct <- function(U,V, type=c("product", "tensor", "strong"), separator='
 
 
 #' @title Make each space in a node apprear only once
-#' 
+#'
 #' @description Reduce a graph to have unique node names
-#' 
+#'
 #' @details Note this is a string based operation. Node names must not contain
 #'   the separator character!
-#'   
+#'
 #' @param graph graph of class loongraph
 #' @template graph_separator
-#' 
+#'
 #' @template return_loongraph
-#'       
+#'
 #' @export
-#' 
+#'
 #' @examples
 #' G <- completegraph(nodes=LETTERS[1:4])
 #' LG <- linegraph(G)
-#' 
+#'
 #' LLG <- linegraph(LG)
-#' 
+#'
 #' graphreduce(LLG)
-#' 
+#'
 #' if (requireNamespace("Rgraphviz", quietly = TRUE)) {
 #'   plot(graphreduce(LLG))
 #' }
-#' 
+#'
 graphreduce <- function(graph, separator) {
 
     if(!is(graph, 'loongraph'))
         stop('graphreduce is only implemented for objects of class loongraph')
-    
+
     if (graph$isDirected == TRUE)
         stop("graphreduce is not implemented for directed graphs")
-    
-    
+
+
     if (missing(separator)) {
         if (!is.null(attr(graph, "separator"))) {
             separator <- attr(graph, "separator")
@@ -465,30 +465,30 @@ graphreduce <- function(graph, separator) {
             stop('separator not known')
         }
     }
-    
-    
+
+
     nodes_reduce <- function(nodes) {
         nodes_vars <- lapply(strsplit(nodes, separator, fixed = TRUE), unique)
         unlist(lapply(nodes_vars, function(vars)paste(sort(vars), collapse = separator)))
     }
-    
+
     ## quick way to get to result of removing duplicate edges
-    
+
     from <- nodes_reduce(graph$from)
     to <- nodes_reduce(graph$to)
-    
+
     sep_nodes <- paste0(separator, separator)
     if(any(grepl(sep_nodes, from)) || any(grepl(sep_nodes, to)))
        stop(paste0('nodes can not contain the string (twice the separator):', sep_nodes ))
-    
+
     edges <- unique(apply(cbind(from,to), 1, function(nodes)paste(sort(nodes), collapse = sep_nodes)))
-    
+
     edges_split <- strsplit(edges, sep_nodes, fixed = TRUE)
-    
+
     new_from <- vapply(edges_split, function(x)x[1], FUN.VALUE = character(1))
     new_to <- vapply(edges_split, function(x)x[2], FUN.VALUE = character(1))
-    
-    
+
+
     loongraph(nodes = unique(nodes_reduce(graph$nodes)),
               from = new_from,
               to = new_to,
@@ -497,25 +497,25 @@ graphreduce <- function(graph, separator) {
 
 
 #' @title Create a n-d transition graph
-#' 
+#'
 #' @description A n-d transition graph has k-d nodes and all edges that connect
 #'   two nodes that from a n-d subspace
-#' 
+#'
 #' @param nodes node names of graph
 #' @param n integer, dimension an edge should represent
 #' @param separator character that separates spaces in node names
-#' 
+#'
 #' @templateVar page learn_R_display_graph.html
 #' @templateVar section graph-utilities
 #' @template see_l_help
-#' @details 
-#'   
-#' 
+#' @details
+#'
+#'
 #' @template return_loongraph
-#' 
+#'
 #' @export
-#' 
-#' @examples 
+#'
+#' @examples
 #' g <- ndtransitiongraph(nodes=c('A:B', 'A:F', 'B:C', 'B:F'), n=3, separator=':')
 ndtransitiongraph <- function(nodes, n, separator=":") {
 
