@@ -172,12 +172,14 @@ package ifneeded loon 1.2.3 "
    [list source [file join $dir library/oo_GraphInspectorNodesAnalysis.tcl]]
    [list source [file join $dir library/x_resize.tcl]]
    [list source [file join $dir library/applySelectionLogic.tcl]]
+   [list source [file join $dir library/oo_SerialaxesLayer.tcl]]
    [list source [file join $dir library/x_context.tcl]]
    [list source [file join $dir library/x_map.tcl]]
    [list source [file join $dir library/data.tcl]]
    [list source [file join $dir library/x_synchronize.tcl]]
    [list source [file join $dir library/setMetaKeys.tcl]]
    [list source [file join $dir library/oo_GraphInspector.tcl]]
+   [list source [file join $dir library/oo_SerialaxesInspector.tcl]]
    [list source [file join $dir library/x_graphutils.tcl]]
    [list source [file join $dir library/oo_Serialaxes_Widget.tcl]]
    [list source [file join $dir library/toplevel.tcl]]
